@@ -172,15 +172,12 @@ ffeequiv_layout_local_ (ffeequiv eq)
 
     }
 
-  if (!ok)
+  if (!ok || (sr == NULL))
     {
-      ffeequiv_kill (eq);	/* Fully processed, no longer needed. */
+      ffeequiv_kill (eq);
       return;
     }
 
-  if (sr == NULL)
-    return;			/* No syms in lists, return. */
-
   /* We've got work to do, so make the LOCAL storage object that'll hold all
      the equivalenced vars inside it. */
 
@@ -197,9 +194,12 @@ ffeequiv_layout_local_ (ffeequiv eq)
   ffestorag_set_basictype (st, ffesymbol_basictype (sr));
   ffestorag_set_kindtype (st, ffesymbol_kindtype (sr));
   ffestorag_set_typesymbol (st, sr);
-  ffestorag_set_save (st, ffeequiv_save (eq));
-  if (ffesymbol_save (sr))
+  ffestorag_set_is_save (st, ffeequiv_is_save (eq));
+  if (ffesymbol_is_save (sr))
     ffestorag_update_save (st);
+  ffestorag_set_is_init (st, ffeequiv_is_init (eq));
+  if (ffesymbol_is_init (sr))
+    ffestorag_update_init (st);
 
   /* Make the EQUIV storage object for the root symbol. */
 
@@ -229,13 +229,16 @@ ffeequiv_layout_local_ (ffeequiv eq)
   ffestorag_set_basictype (rst, ffesymbol_basictype (sr));
   ffestorag_set_kindtype (rst, ffesymbol_kindtype (sr));
   ffestorag_set_typesymbol (rst, sr);
-  ffestorag_set_save (rst, FALSE);	/* Assume FALSE, then... */
-  if (ffestorag_save (st))	/* ...update to TRUE if needed. */
+  ffestorag_set_is_save (rst, FALSE);	/* Assume FALSE, then... */
+  if (ffestorag_is_save (st))	/* ...update to TRUE if needed. */
     ffestorag_update_save (rst);
+  ffestorag_set_is_init (rst, FALSE);	/* Assume FALSE, then... */
+  if (ffestorag_is_init (st))	/* ...update to TRUE if needed. */
+    ffestorag_update_init (rst);
   ffestorag_set_size (st, size);
   ffesymbol_set_storage (sr, rst);
   ffesymbol_signal_unreported (sr);
-  init = (ffesymbol_init (sr) != NULL) || (ffesymbol_accretion (sr) != NULL);
+  init = ffesymbol_is_init (sr);
 
   /* Now that we know the root (offset=0) symbol, revisit all the lists and
      do the actual storage allocation.	Keep doing this until we've gone
@@ -379,9 +382,12 @@ ffeequiv_layout_local_ (ffeequiv eq)
 		  ffestorag_set_basictype (vst, ffesymbol_basictype (sv));
 		  ffestorag_set_kindtype (vst, ffesymbol_kindtype (sv));
 		  ffestorag_set_typesymbol (vst, sv);
-		  ffestorag_set_save (vst, FALSE);	/* Assume FALSE... */
-		  if (ffestorag_save (st))	/* ...update TRUE */
+		  ffestorag_set_is_save (vst, FALSE);	/* Assume FALSE... */
+		  if (ffestorag_is_save (st))	/* ...update TRUE */
 		    ffestorag_update_save (vst);	/* if needed. */
+		  ffestorag_set_is_init (vst, FALSE);	/* Assume FALSE... */
+		  if (ffestorag_is_init (st))	/* ...update TRUE */
+		    ffestorag_update_init (vst);	/* if needed. */
 		  if (!ffetarget_offset_add (&size, var_offset, size))
 		    /* Find one size of equiv area, complain if overflow. */
 		    ffetarget_offset_overflow (ffesymbol_text (s));
@@ -392,8 +398,7 @@ ffeequiv_layout_local_ (ffeequiv eq)
 		  ffesymbol_signal_unreported (sv);
 		  ffestorag_update (st, sv, ffesymbol_basictype (sv),
 				    ffesymbol_kindtype (sv));
-		  if ((ffesymbol_init (sv) != NULL)
-		      || (ffesymbol_accretion (sv) != NULL))
+		  if (ffesymbol_is_init (sv))
 		    init = TRUE;
 		}
 	      else
@@ -698,23 +703,24 @@ ffeequiv_add (ffeequiv eq, ffebld list, ffelexToken t)
       if (ffesymbol_common (symbol) == NULL)	/* Is symbol in a COMMON
 						   area? */
 	{			/* No (at least not yet). */
-	  if ((ffesymbol_init (symbol) != NULL)
-	      || (ffesymbol_accretion (symbol) != NULL))
-	    eq->init = TRUE;
-	  if (ffesymbol_save (symbol))
+	  if (ffesymbol_is_save (symbol))
 	    ffeequiv_update_save (eq);	/* EQUIVALENCE has >=1 SAVEd entity. */
+	  if (ffesymbol_is_init (symbol))
+	    ffeequiv_update_init (eq);	/* EQUIVALENCE has >=1 init'd entity. */
 	  continue;		/* Nothing more to do here. */
 	}
 
 #if FFEGLOBAL_ENABLED
-      if ((ffesymbol_init (symbol) != NULL)
-	  || (ffesymbol_accretion (symbol) != NULL))
+      if (ffesymbol_is_init (symbol))
 	ffeglobal_init_common (ffesymbol_common (symbol), t);
 #endif
 
-      if (ffesymbol_save (ffesymbol_common (symbol)))
+      if (ffesymbol_is_save (ffesymbol_common (symbol)))
 	ffeequiv_update_save (eq);	/* EQUIVALENCE is in a SAVEd COMMON
 					   block. */
+      if (ffesymbol_is_init (ffesymbol_common (symbol)))
+	ffeequiv_update_init (eq);	/* EQUIVALENCE is in a init'd COMMON
+					   block. */
 
       if (ffeequiv_common (eq) == NULL)	/* Is COMMON involved already? */
 	/* No, but there is now. */
@@ -743,7 +749,7 @@ void
 ffeequiv_dump (ffeequiv eq)
 {
   if (ffeequiv_common (eq) != NULL)
-    fprintf (stdout, "(common %s) ", ffesymbol_text (ffeequiv_common (eq)));
+    fprintf (dmpout, "(common %s) ", ffesymbol_text (ffeequiv_common (eq)));
   ffebld_dump (ffeequiv_list (eq));
 }
 
@@ -956,9 +962,12 @@ ffeequiv_layout_cblock (ffestorag st)
 		      ffestorag_set_basictype (vst, ffesymbol_basictype (sv));
 		      ffestorag_set_kindtype (vst, ffesymbol_kindtype (sv));
 		      ffestorag_set_typesymbol (vst, sv);
-		      ffestorag_set_save (vst, FALSE);	/* Assume FALSE... */
-		      if (ffestorag_save (st))	/* ...update TRUE */
+		      ffestorag_set_is_save (vst, FALSE);	/* Assume FALSE... */
+		      if (ffestorag_is_save (st))	/* ...update TRUE */
 			ffestorag_update_save (vst);	/* if needed. */
+		      ffestorag_set_is_init (vst, FALSE);	/* Assume FALSE... */
+		      if (ffestorag_is_init (st))	/* ...update TRUE */
+			ffestorag_update_init (vst);	/* if needed. */
 		      if (!ffetarget_offset_add (&size, var_offset, size))
 			/* Find one size of common block, complain if
 			   overflow. */
@@ -971,8 +980,7 @@ ffeequiv_layout_cblock (ffestorag st)
 		      ffesymbol_signal_unreported (sv);
 		      ffestorag_update (st, sv, ffesymbol_basictype (sv),
 					ffesymbol_kindtype (sv));
-		      if ((ffesymbol_init (sv) != NULL)
-			  || (ffesymbol_accretion (sv) != NULL))
+		      if (ffesymbol_is_init (sv))
 			init = TRUE;
 		    }
 		  else
@@ -1043,20 +1051,26 @@ ffeequiv_merge (ffeequiv eq1, ffeequiv eq2, ffelexToken t)
 
   /* If the victim object has any init'ed entities, so does the new object. */
 
-  if (eq2->init)
-    eq1->init = TRUE;
+  if (eq2->is_init)
+    eq1->is_init = TRUE;
 
 #if FFEGLOBAL_ENABLED
-  if (eq1->init && (ffeequiv_common (eq1) != NULL))
+  if (eq1->is_init && (ffeequiv_common (eq1) != NULL))
     ffeglobal_init_common (ffeequiv_common (eq1), t);
 #endif
 
   /* If the victim object has any SAVEd entities, then the new object has
      some. */
 
-  if (ffeequiv_save (eq2))
+  if (ffeequiv_is_save (eq2))
     ffeequiv_update_save (eq1);
 
+  /* If the victim object has any init'd entities, then the new object has
+     some. */
+
+  if (ffeequiv_is_init (eq2))
+    ffeequiv_update_init (eq1);
+
   /* Adjust all the symbols in the list of lists of equivalences for the
      victim equivalence object so they point to the new merged object
      instead. */
@@ -1113,8 +1127,8 @@ ffeequiv_new ()
   eq->previous = ffeequiv_list_.last;
   ffeequiv_set_common (eq, NULL);	/* No COMMON area yet. */
   ffeequiv_set_list (eq, NULL);	/* No list of lists of equivalences yet. */
-  ffeequiv_set_save (eq, FALSE);
-  eq->init = FALSE;
+  ffeequiv_set_is_save (eq, FALSE);
+  ffeequiv_set_is_init (eq, FALSE);
   eq->next->previous = eq;
   eq->previous->next = eq;
 
@@ -1156,6 +1170,65 @@ again:				/* :::::::::::::::::::: */
     }
 }
 
+/* ffeequiv_update_init -- Update the INIT flag for the area to TRUE
+
+   ffeequiv eq;
+   ffeequiv_update_init(eq);
+
+   If the INIT flag for the <eq> object is already set, return.	 Else,
+   set it TRUE and call ffe*_update_init for all objects contained in
+   this one.  */
+
+void
+ffeequiv_update_init (ffeequiv eq)
+{
+  ffebld list;			/* Current list in list of lists. */
+  ffebld item;			/* Current item in current list. */
+  ffebld expr;			/* Expression in head of current item. */
+
+  if (eq->is_init)
+    return;
+
+  eq->is_init = TRUE;
+
+  if ((eq->common != NULL)
+      && !ffesymbol_is_init (eq->common))
+    ffesymbol_update_init (eq->common);	/* Shouldn't be needed. */
+
+  for (list = eq->list; list != NULL; list = ffebld_trail (list))
+    {
+      for (item = ffebld_head (list); item != NULL; item = ffebld_trail (item))
+	{
+	  expr = ffebld_head (item);
+
+	again:			/* :::::::::::::::::::: */
+
+	  switch (ffebld_op (expr))
+	    {
+	    case FFEBLD_opANY:
+	      break;
+
+	    case FFEBLD_opSYMTER:
+	      if (!ffesymbol_is_init (ffebld_symter (expr)))
+		ffesymbol_update_init (ffebld_symter (expr));
+	      break;
+
+	    case FFEBLD_opARRAYREF:
+	      expr = ffebld_left (expr);
+	      goto again;	/* :::::::::::::::::::: */
+
+	    case FFEBLD_opSUBSTR:
+	      expr = ffebld_left (expr);
+	      goto again;	/* :::::::::::::::::::: */
+
+	    default:
+	      assert ("bad op for ffeequiv_update_init" == NULL);
+	      break;
+	    }
+	}
+    }
+}
+
 /* ffeequiv_update_save -- Update the SAVE flag for the area to TRUE
 
    ffeequiv eq;
@@ -1172,12 +1245,13 @@ ffeequiv_update_save (ffeequiv eq)
   ffebld item;			/* Current item in current list. */
   ffebld expr;			/* Expression in head of current item. */
 
-  if (eq->save)
+  if (eq->is_save)
     return;
 
-  eq->save = TRUE;
+  eq->is_save = TRUE;
 
-  if (eq->common != NULL)
+  if ((eq->common != NULL)
+      && !ffesymbol_is_save (eq->common))
     ffesymbol_update_save (eq->common);	/* Shouldn't be needed. */
 
   for (list = eq->list; list != NULL; list = ffebld_trail (list))
@@ -1194,7 +1268,8 @@ ffeequiv_update_save (ffeequiv eq)
 	      break;
 
 	    case FFEBLD_opSYMTER:
-	      ffesymbol_update_save (ffebld_symter (expr));
+	      if (!ffesymbol_is_save (ffebld_symter (expr)))
+		ffesymbol_update_save (ffebld_symter (expr));
 	      break;
 
 	    case FFEBLD_opARRAYREF:
