@@ -52,19 +52,27 @@ the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
    `gcc' itself.  */
 
 #include "config.j"
+#include <sys/types.h>
+#include <errno.h>
+
+#ifndef _WIN32
+#include <sys/file.h>   /* May get R_OK, etc. on some systems.  */
+#else
+#include <process.h>
+#endif
+
 #ifdef __STDC__
 #include <stdarg.h>
 #else
 #include <varargs.h>
 #endif
 #include <stdio.h>
-#include <sys/types.h>
-#ifndef _WIN32
-#include <sys/file.h>   /* May get R_OK, etc. on some systems.  */
-#else
-#include <process.h>
+
+#ifndef R_OK
+#define R_OK 4
+#define W_OK 2
+#define X_OK 1
 #endif
-#include <errno.h>
 
 /* Defined to the name of the compiler; if using a cross compiler, the
    Makefile should compile this file with the proper name
@@ -86,6 +94,15 @@ the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 #endif
 #endif
 
+/* Add prototype support.  */
+#ifndef PROTO
+#if defined (USE_PROTOTYPES) ? USE_PROTOTYPES : defined (__STDC__)
+#define PROTO(ARGS) ARGS
+#else
+#define PROTO(ARGS) ()
+#endif
+#endif
+
 #ifndef VPROTO
 #ifdef __STDC__
 #define PVPROTO(ARGS)		ARGS
@@ -98,6 +115,59 @@ the Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 #endif
 #endif
 
+/* Define a generic NULL if one hasn't already been defined.  */
+
+#ifndef NULL
+#define NULL 0
+#endif
+
+/* Define O_RDONLY if the system hasn't defined it for us. */
+#ifndef O_RDONLY
+#define O_RDONLY 0
+#endif
+
+#ifndef GENERIC_PTR
+#if defined (USE_PROTOTYPES) ? USE_PROTOTYPES : defined (__STDC__)
+#define GENERIC_PTR void *
+#else
+#define GENERIC_PTR char *
+#endif
+#endif
+
+#ifndef NULL_PTR
+#define NULL_PTR ((GENERIC_PTR)0)
+#endif
+
+#ifdef USG
+#define vfork fork
+#endif /* USG */
+
+/* On MSDOS, write temp files in current dir
+   because there's no place else we can expect to use.  */
+#ifdef __MSDOS__
+#ifndef P_tmpdir
+#define P_tmpdir "."
+#endif
+#endif
+
+/* By default there is no special suffix for executables.  */
+#ifndef EXECUTABLE_SUFFIX
+#define EXECUTABLE_SUFFIX ""
+#endif
+
+/* By default, colon separates directories in a path.  */
+#ifndef PATH_SEPARATOR
+#define PATH_SEPARATOR ':'
+#endif
+
+#ifndef DIR_SEPARATOR
+#define DIR_SEPARATOR '/'
+#endif
+
+static char dir_separator_str[] = {DIR_SEPARATOR, 0};
+
+extern char *getenv ();
+
 #ifndef errno
 extern int errno;
 #endif
@@ -147,7 +217,7 @@ typedef enum
   OPTION_			/* Unrecognized or unimportant. */
 } Option;
 
-/* THE FOLLOWING COMES STRAIGHT FROM gcc-2.6.3/gcc.c:  */
+/* THE FOLLOWING COMES STRAIGHT FROM gcc-2.7.0/gcc.c:  */
 
 /* This defines which switch letters take arguments.  */
 
@@ -155,7 +225,7 @@ typedef enum
 #define SWITCH_TAKES_ARG(CHAR)      \
   ((CHAR) == 'D' || (CHAR) == 'U' || (CHAR) == 'o' \
    || (CHAR) == 'e' || (CHAR) == 'T' || (CHAR) == 'u' \
-   || (CHAR) == 'I' || (CHAR) == 'm' \
+   || (CHAR) == 'I' || (CHAR) == 'm' || (CHAR) == 'x' \
    || (CHAR) == 'L' || (CHAR) == 'A')
 #endif
 
@@ -173,7 +243,23 @@ typedef enum
 #define WORD_SWITCH_TAKES_ARG(STR) DEFAULT_WORD_SWITCH_TAKES_ARG (STR)
 #endif
 
-/* END OF STUFF FROM gcc-2.6.3/gcc.c.  */
+/* This is the common prefix we use to make temp file names.
+   It is chosen once for each run of this program.
+   It is substituted into a spec by %g.
+   Thus, all temp file names contain this prefix.
+   In practice, all temp file names start with this prefix.
+
+   This prefix comes from the envvar TMPDIR if it is defined;
+   otherwise, from the P_tmpdir macro if that is defined;
+   otherwise, in /usr/tmp or /tmp.  */
+
+static char *temp_filename;
+
+/* Length of the prefix.  */
+
+static int temp_filename_length;
+
+/* END OF STUFF FROM gcc-2.7.0/gcc.c.  */
 
 char *
 my_strerror(e)
@@ -290,23 +376,42 @@ xmalloc (size)
   return value;
 }
 
-/* Return a newly-allocated string whose contents concatenate those
-   of s1, s2, s3.  */
 static char *
-concat (s1, s2, s3)
-     char *s1, *s2, *s3;
+concat (s1, s2)
+     char *s1, *s2;
 {
-  int len1 = strlen (s1), len2 = strlen (s2), len3 = strlen (s3);
-  char *result = xmalloc (len1 + len2 + len3 + 1);
+  int len1 = strlen (s1);
+  int len2 = strlen (s2);
+  char *result = xmalloc (len1 + len2 + 1);
 
   strcpy (result, s1);
   strcpy (result + len1, s2);
-  strcpy (result + len1 + len2, s3);
-  *(result + len1 + len2 + len3) = 0;
+  *(result + len1 + len2) = 0;
 
   return result;
 }
 
+static char *
+concat3 (s1, s2, s3)
+     char *s1, *s2, *s3;
+{
+  return concat (concat (s1, s2), s3);
+}
+
+static char *
+concat4 (s1, s2, s3, s4)
+     char *s1, *s2, *s3, *s4;
+{
+  return concat (concat (s1, s2), concat (s3, s4));
+}
+
+static char *
+concat6 (s1, s2, s3, s4, s5, s6)
+     char *s1, *s2, *s3, *s4, *s5, *s6;
+{
+  return concat3 (concat (s1, s2), concat (s3, s4), concat (s5, s6));
+}
+
 static void
 pfatal_with_name (name)
      char *name;
@@ -314,24 +419,19 @@ pfatal_with_name (name)
   char *s;
 
   if (errno < sys_nerr)
-    s = concat ("%s: ", sys_errlist[errno], "");
+    s = concat3 ("%s: ", sys_errlist[errno], "");
   else
     s = "cannot open %s";
   fatal (s, name);
 }
 
-#ifdef __MSDOS__
-/* This is the common prefix we use to make temp file names.  */
-char *temp_filename;
+/* Compute a string to use as the base of all temporary file names.
+   It is substituted for %g.  */
 
-/* Length of the prefix.  */
-int temp_filename_length;
-
-/* Compute a string to use as the base of all temporary file names.  */
 static char *
 choose_temp_base_try (try, base)
-char *try;
-char *base;
+     char *try;
+     char *base;
 {
   char *rv;
   if (base)
@@ -359,19 +459,23 @@ choose_temp_base ()
   base = choose_temp_base_try (P_tmpdir, base);
 #endif
 
-  base = choose_temp_base_try ("/usr/tmp", base);
-  base = choose_temp_base_try ("/tmp", base);
-
+  base = choose_temp_base_try (concat4 (dir_separator_str, "usr", 
+                                        dir_separator_str, "tmp"), 
+                                base);
+  base = choose_temp_base_try (concat (dir_separator_str, "tmp"), base);
+ 
   /* If all else fails, use the current directory! */
   if (base == (char *)0)
-    base = "./";
+    base = concat (".", dir_separator_str);
 
   len = strlen (base);
-  temp_filename = xmalloc (len + sizeof("/ccXXXXXX"));
+  temp_filename = xmalloc (len + strlen (concat (dir_separator_str, 
+                                                 "gfXXXXXX")) + 1);
   strcpy (temp_filename, base);
-  if (len > 0 && temp_filename[len-1] != '/')
-    temp_filename[len++] = '/';
-  strcpy (temp_filename + len, "ccXXXXXX");
+  if (len > 0 && temp_filename[len-1] != '/'
+      && temp_filename[len-1] != DIR_SEPARATOR)
+    temp_filename[len++] = DIR_SEPARATOR;
+  strcpy (temp_filename + len, "gfXXXXXX");
 
   mktemp (temp_filename);
   temp_filename_length = strlen (temp_filename);
@@ -379,6 +483,7 @@ choose_temp_base ()
     abort ();
 }
 
+#ifdef __MSDOS__
 static void
 perror_exec (name)
      char *name;
@@ -386,8 +491,8 @@ perror_exec (name)
   char *s;
 
   if (errno < sys_nerr)
-    s = concat ("installation problem, cannot exec %s: ",
-		my_strerror( errno ), "");
+    s = concat3 ("installation problem, cannot exec %s: ",
+		 my_strerror( errno ), "");
   else
     s = "installation problem, cannot exec %s";
   error (s, name);
@@ -1016,9 +1121,11 @@ main (argc, argv)
     }
   else if (verbose && add_version_magic)
     {
+      choose_temp_base ();
+
       append_arg ("-fnull-version");
       append_arg ("-o");
-      append_arg ("/dev/null");
+      append_arg (temp_filename);
       append_arg ("-xf77-cpp-input");
       append_arg ("/dev/null");
       append_arg ("-xnone");
