@@ -1,9 +1,9 @@
-static char junk[] = "\n@(#) LIBI77 VERSION pjw,dmg-mods 29 Aug. 1995\n";
+static char junk[] = "\n@(#) LIBI77 VERSION pjw,dmg-mods 19960315\n";
 
 /*
 */
 
-char __G77_LIBI77_VERSION__[] = "0.5.16";
+char __G77_LIBI77_VERSION__[] = "0.5.18";
 
 /*
 2.01	$ format added
@@ -216,6 +216,22 @@ wrtfmt.c:
 		 in internal writes whose last item is written to
 		 an earlier position than some previous item. */
 /* 29 Aug. 1995: backspace.c: adjust MSDOS logic. */
+/* 6 Sept. 1995: Adjust namelist input to treat a subscripted name
+		 whose subscripts do not involve colons similarly
+		 to the name without a subscript: accept several
+		 values, stored in successive elements starting at
+		 the indicated subscript.  Adjust namelist output
+		 to quote character strings (avoiding confusion with
+		 arrays of character strings).  Adjust f_init calls
+		 for people who don't use libF77's main(); now open and
+		 namelist read statements invoke f_init if needed. */
+/* 7 Sept. 1995: Fix some bugs with -DAllow_TYQUAD (for integer*8).
+		 Add -DNo_Namelist_Comments lines to rsne.c. */
+/* 5 Oct. 1995:  wrtfmt.c: fix bug with t editing (f__cursor was not
+		 always zeroed in mv_cur). */
+/* 11 Oct. 1995: move defs of f__hiwater, f__svic, f__icptr from wrtfmt.c
+		 to err.c */
+/* 15 Mar. 1996: lread.c, rsfe.c: honor END= in READ stmt with empty iolist */
 
 #include "stdio.h"
 
