@@ -101,37 +101,37 @@ ffestorag_dump (ffestorag s)
 {
   if (s == NULL)
     {
-      fprintf (stdout, "(no storage object)");
+      fprintf (dmpout, "(no storage object)");
       return;
     }
 
   switch (s->type)
     {
     case FFESTORAG_typeCBLOCK:
-      fprintf (stdout, "CBLOCK ");
+      fprintf (dmpout, "CBLOCK ");
       break;
 
     case FFESTORAG_typeCOMMON:
-      fprintf (stdout, "COMMON ");
+      fprintf (dmpout, "COMMON ");
       break;
 
     case FFESTORAG_typeLOCAL:
-      fprintf (stdout, "LOCAL ");
+      fprintf (dmpout, "LOCAL ");
       break;
 
     case FFESTORAG_typeEQUIV:
-      fprintf (stdout, "EQUIV ");
+      fprintf (dmpout, "EQUIV ");
       break;
 
     default:
-      fprintf (stdout, "?%d? ", s->type);
+      fprintf (dmpout, "?%d? ", s->type);
       break;
     }
 
   if (s->symbol != NULL)
-    fprintf (stdout, "\"%s\" ", ffesymbol_text (s->symbol));
+    fprintf (dmpout, "\"%s\" ", ffesymbol_text (s->symbol));
 
-  fprintf (stdout, "at %" ffetargetOffset_f "u size %" ffetargetOffset_f
+  fprintf (dmpout, "at %" ffetargetOffset_f "u size %" ffetargetOffset_f
 	   "u, align loc%%%"
 	   ffetargetAlign_f "u=%" ffetargetAlign_f "u, bt=%s, kt=%s",
 	   s->offset,
@@ -143,16 +143,16 @@ ffestorag_dump (ffestorag s)
     {
       ffestorag sq;
 
-      fprintf (stdout, " with equivs");
+      fprintf (dmpout, " with equivs");
       for (sq = s->equivs_.first;
 	   sq != (ffestorag) &s->equivs_.first;
 	   sq = ffestorag_next_ (sq))
 	{
 	  if (ffestorag_previous_ (sq) == (ffestorag) &s->equivs_.first)
-	    fputc (' ', stdout);
+	    fputc (' ', dmpout);
 	  else
-	    fputc (',', stdout);
-	  fprintf (stdout, "%s", ffesymbol_text (ffestorag_symbol (sq)));
+	    fputc (',', dmpout);
+	  fprintf (dmpout, "%s", ffesymbol_text (ffestorag_symbol (sq)));
 	}
     }
 }
@@ -234,8 +234,14 @@ ffestorag_exec_layout (ffesymbol s)
 	  if (ffesymbol_rank (s) == 0)
 	    num_elements = 1;
 	  else
-	    num_elements = ffebld_constant_integerdefault (ffebld_conter
-						 (ffesymbol_arraysize (s)));
+	    {
+	      if (ffebld_op (ffesymbol_arraysize (s))
+		  != FFEBLD_opCONTER)
+		return;	/* An adjustable local array, just like a dummy. */
+	      num_elements
+		= ffebld_constant_integerdefault (ffebld_conter
+						  (ffesymbol_arraysize (s)));
+	    }
 	  ffetarget_layout (ffesymbol_text (s), &alignment, &modulo,
 			    &size, ffesymbol_basictype (s),
 			    ffesymbol_kindtype (s), ffesymbol_size (s),
@@ -253,9 +259,10 @@ ffestorag_exec_layout (ffesymbol s)
 	  st->basic_type = ffesymbol_basictype (s);
 	  st->kind_type = ffesymbol_kindtype (s);
 	  st->type_symbol = s;
-	  st->save = ffesymbol_save (s);
+	  st->is_save = ffesymbol_is_save (s);
+	  st->is_init = ffesymbol_is_init (s);
 	  ffesymbol_set_storage (s, st);
-	  if (ffesymbol_init (s) != NULL)
+	  if (ffesymbol_is_init (s))
 	    ffecom_notify_init_symbol (s);	/* Init completed before, but
 						   we didn't have a storage
 						   object for it; maybe back
@@ -310,10 +317,11 @@ ffestorag_exec_layout (ffesymbol s)
 	  st->kind_type = FFEINFO_kindtypeNONE;
 	  st->type_symbol = NULL;
 	}
-      st->save = ffesymbol_save (s);
+      st->is_save = ffesymbol_is_save (s);
+      st->is_init = ffesymbol_is_init (s);
       if (!ffe_is_mainprog ())
 	ffeglobal_save_common (s,
-			       st->save || ffe_is_saveall (),
+			       st->is_save || ffe_is_saveall (),
 			       ffesymbol_where_line (s),
 			       ffesymbol_where_column (s));
       ffesymbol_set_storage (s, st);
@@ -376,13 +384,13 @@ ffestorag_exec_layout (ffesymbol s)
 	  stv->basic_type = ffesymbol_basictype (var);
 	  stv->kind_type = ffesymbol_kindtype (var);
 	  stv->type_symbol = var;
-	  stv->save = st->save;
+	  stv->is_save = st->is_save;
+	  stv->is_init = st->is_init;
 	  ffesymbol_set_storage (var, stv);
 	  ffesymbol_signal_unreported (var);
 	  ffestorag_update (st, var, ffesymbol_basictype (var),
 			    ffesymbol_kindtype (var));
-	  if ((ffesymbol_init (var) != NULL)
-	      || (ffesymbol_accretion (var) != NULL))
+	  if (ffesymbol_is_init (var))
 	    init = TRUE;	/* Must move inits over to COMMON's
 				   ffestorag. */
 	}
@@ -439,9 +447,9 @@ ffestorag_report ()
       if (s->symbol == NULL)
 	{
 	  ffestorag_reported_ = TRUE;
-	  fputs ("Storage area: ", stdout);
+	  fputs ("Storage area: ", dmpout);
 	  ffestorag_dump (s);
-	  fputc ('\n', stdout);
+	  fputc ('\n', dmpout);
 	}
     }
 }
@@ -502,6 +510,34 @@ ffestorag_update (ffestorag s, ffesymbol sym, ffeinfoBasictype bt,
     }
 }
 
+/* Update INIT flag for storage object.
+
+   If the INIT flag for the <s> object is already TRUE, return.	 Else,
+   set it to TRUE and call ffe*_update_init for all contained objects.	*/
+
+void
+ffestorag_update_init (ffestorag s)
+{
+  ffestorag sq;
+
+  if (s->is_init)
+    return;
+
+  s->is_init = TRUE;
+
+  if ((s->symbol != NULL)
+      && !ffesymbol_is_init (s->symbol))
+    ffesymbol_update_init (s->symbol);
+
+  for (sq = s->equivs_.first;
+       sq != (ffestorag) &s->equivs_.first;
+       sq = ffestorag_next_ (sq))
+    {
+      if (!sq->is_init)
+	ffestorag_update_init (sq);
+    }
+}
+
 /* Update SAVE flag for storage object.
 
    If the SAVE flag for the <s> object is already TRUE, return.	 Else,
@@ -512,16 +548,20 @@ ffestorag_update_save (ffestorag s)
 {
   ffestorag sq;
 
-  if (s->save)
+  if (s->is_save)
     return;
 
-  s->save = TRUE;
+  s->is_save = TRUE;
 
-  if (s->symbol != NULL)
+  if ((s->symbol != NULL)
+      && !ffesymbol_is_save (s->symbol))
     ffesymbol_update_save (s->symbol);
 
   for (sq = s->equivs_.first;
        sq != (ffestorag) &s->equivs_.first;
        sq = ffestorag_next_ (sq))
-    ffestorag_update_save (sq);
+    {
+      if (!sq->is_save)
+	ffestorag_update_save (sq);
+    }
 }
